# $Header: /cvs/www/mt/topicon/topicon.pl,v 1.12 2003/04/27 13:43:20 john Exp $
#--------------------------------------------------
# topicon.pl - Movable Type plugin
#
# Version: 1.20 
# Date: 2003-04-08
#
#--------------------------------------------------
# Copyright (c) 2002-2003 by John Addis
# All Rights Reserved
#
# This program is free software, distributed under
# a BSD-style license. Please see the LICENSE file
# for details.
#--------------------------------------------------
# Contributors
# ------------
# John Addis <john@a42.net>
# Timothy Carl Buchheim <tim@buchheim.name>
#
#--------------------------------------------------
package MT::Plugin::TopicIcon;
use vars qw($VERSION);
$VERSION=1.2.0;

use strict;
use MT::Template::Context;
MT::Template::Context->add_tag(TopicIcon => \&MT::Plugin::TopicIcon::TopicIcon);
MT::Template::Context->add_tag(TopicIconURL => \&MT::Plugin::TopicIcon::TopicIconURL);

use File::Spec::Functions qw(catfile);

use FindBin qw($Bin);
use IO::File;

#--------------------------------------------------
# Configuration
#--------------------------------------------------

# This value tells us where to find the config file. You
# should not change it unless the FindBin mechanism does
# not work for you.
my $CONFIG="$Bin/plugins/topicon.cfg";

my %CFG = (
    'extensions'     => "png gif jpg",
    'blog_icon_path' => "Icons",
    'blog_icon_url'  => undef,    # Later, default it to path.
    'site_icon_path' => undef,    # absolute path
    'site_icon_url'  => undef,    # absolute url

);


# Now, read the config file.
_ReadConfigFile();
$CFG{blog_icon_url} = $CFG{blog_icon_path} unless defined($CFG{blog_icon_url});

#--------------------------------------------------


sub TopicIcon
{
    my ($ctx, $args) = @_;
    my $deftext = _PreprocessArgs( $ctx, $args );
    my $url = TopicIconURL( $ctx, $args );
    if ($url)
    {
        my $tag = qq/<img src="$url"/;
        foreach(sort keys %$args)
        {
            $tag .= qq/ $_="$args->{$_}"/;
        }
        return "$tag />";
    }

    return "$deftext";
}

sub TopicIconURL
{
    my ($ctx, $args) = @_;

    my $deftext = _PreprocessArgs( $ctx, $args )||"";

    my $cat = _GetCategory( $ctx ) || return $deftext;
    my $blog = $ctx->stash( 'blog' );

    # Form the file name
    return _BuildURL( $cat->label, $blog->site_path, $blog->site_url, $args ) || $deftext;
}

sub _GetCategory
{
    my $ctx = shift;
    my $cat = $ctx->stash( 'category' );
    if (not defined($cat))
    {
        my $entry = $ctx->stash( 'entry' );
        $cat = $entry->category if (defined($entry))
    }

    return $cat;
}

sub _PreprocessArgs
{
    my($ctx, $args) = @_;

    my %percent = ( '%' => '%', 'c' => "", 'C' => "", );
    my $cat = _GetCategory( $ctx );
    $percent{C} = $percent{c} = $cat->label if defined($cat);

    foreach(keys %$args)
    {
        $args->{$_} =~ s/%(.)/$percent{$1}||""/ge;
    }

    my $deftext = "";
    if (exists $args->{deftext})
    {
        $deftext = $args->{deftext};
        delete $args->{deftext};
    }

    return $deftext;
}



sub _BuildURL
{
    my ($label, $sitepath, $siteurl, $args) = @_;

    my $icon = _BuildIconName( $label, $sitepath, $CFG{blog_icon_path} );
    return _BuildIconURL( $icon, $siteurl, $CFG{blog_icon_url} ) if (defined($icon));

    # Didn't find it in the blog-local cache. Look site-wide
    if ($CFG{site_icon_path} and $CFG{site_icon_url})
    {
        $icon = _BuildIconName( $label, $CFG{site_icon_path}, undef );
        return _BuildIconURL( $icon, $CFG{site_icon_url}, undef ) if (defined($icon));
    }

    return undef;
}

sub _BuildIconName
{
    my ($label, $path, $icondir) = @_;
    $label = _Squeeze( $label );
    $icondir |= "";

    my @extensions = split /\s+/, $CFG{extensions};
    foreach(@extensions)
    {
        my $full = catfile( $path, $icondir, $label ) . ".$_";
        return "$label.$_" if ( -f $full );
    }

    return undef;
}

sub _BuildIconURL
{
    my ($icon, $base, $iconurl) = @_;
    $iconurl |= "";

    # Strip leading /'s
    $iconurl =~ s{^/}{};
    $icon    =~ s{^/}{};

    # Strip trailing /'s
    $icon    =~ s{/$}{};
    $iconurl =~ s{/$}{};
    $base    =~ s{/$}{};
    
    # Build up URL
    my $src = "$base";
    $src .= "/$iconurl" if ($iconurl);
    $src .= "/$icon";

    return "$src";
}



sub _Squeeze
{
    my $text = shift;
    $text =~ s/[\'\"]+//g;
    $text =~ s/[^A-Za-z0-9\.\-]+/_/g;
    return lc($text);
}

sub _ReadConfigFile
{
    my $fh = IO::File->new( "< $CONFIG" );
    if (not defined($fh))
    {
        # TODO: Trace/log something
        return;
    }

    my ($var,$val);
    while(<$fh>)
    {
        chomp;
        s/#.*$//;           # Strip comments
        next unless($_);    # Skip blank lines

        # Parse
        ($var, $val) = $_ =~ /([^=]+)=(.*)/;
        next unless defined($var);

        # Strip leading and trailing spaces
        $var = _trim($var);
        $val = _trim($val);

        # Make sure it is a valid variable
        if (exists $CFG{$var})
        {
            $CFG{$var} = $val;
        }
        else
        {
            # TODO: Log/trace something?
        }
    }

    $fh->close;
}

sub _trim
{
    my $x = shift;
    $x =~ s/^\s+//;
    $x =~ s/\s+$//;

    return $x;
}

1;

